"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvaluationRecord = exports.Evaluation = void 0;
const crypto_1 = require("crypto");
const evaluationRecord_object_1 = __importDefault(require("./evaluationRecord.object"));
exports.EvaluationRecord = evaluationRecord_object_1.default;
const wappalyzer_1 = require("@qualweb/wappalyzer");
class Evaluation {
    async getEvaluator(page, url) {
        const [plainHtml, pageTitle, elements, browserUserAgent] = await Promise.all([
            page.evaluate(() => {
                return document.documentElement.outerHTML;
            }),
            page.title(),
            page.$$('*'),
            page.browser().userAgent()
        ]);
        let urlStructure = undefined;
        if (url) {
            urlStructure = this.parseUrl(url, page.url() !== 'about:blank' ? page.url() : url);
        }
        const viewport = page.viewport();
        return {
            name: 'QualWeb',
            description: 'QualWeb is an automatic accessibility evaluator for webpages.',
            version: '3.0.0',
            homepage: 'http://www.qualweb.di.fc.ul.pt/',
            date: new Date().toISOString().replace(/T/, ' ').replace(/\..+/, ''),
            hash: crypto_1.randomBytes(40).toString('hex'),
            url: urlStructure,
            page: {
                viewport: {
                    mobile: viewport === null || viewport === void 0 ? void 0 : viewport.isMobile,
                    landscape: viewport === null || viewport === void 0 ? void 0 : viewport.isLandscape,
                    userAgent: browserUserAgent,
                    resolution: {
                        width: viewport === null || viewport === void 0 ? void 0 : viewport.width,
                        height: viewport === null || viewport === void 0 ? void 0 : viewport.height
                    }
                },
                dom: {
                    html: plainHtml,
                    title: pageTitle,
                    elementCount: elements.length
                }
            }
        };
    }
    async init(page) {
        await page.addScriptTag({
            path: require.resolve('@qualweb/qw-page')
        });
        await page.addScriptTag({
            path: require.resolve('@qualweb/util')
        });
        await page.evaluate(() => {
            window.qwPage = new Module.QWPage(document, window, true);
        });
        await page.evaluate(() => {
            window.DomUtils = Utility.DomUtils;
            window.AccessibilityUtils = Utility.AccessibilityUtils;
        });
    }
    async executeACT(page, sourceHtmlHeadContent, options) {
        await page.addScriptTag({
            path: require.resolve('@qualweb/act-rules')
        });
        await page.evaluate((options) => {
            window.act = new ACT.ACTRules(options);
        }, options);
        await page.keyboard.press('Tab');
        await page.evaluate((sourceHtmlHeadContent) => {
            window.act.validateFirstFocusableElementIsLinkToNonRepeatedContent();
            const parser = new DOMParser();
            const sourceDoc = parser.parseFromString('', 'text/html');
            sourceDoc.head.innerHTML = sourceHtmlHeadContent;
            const elements = sourceDoc.querySelectorAll('meta');
            const metaElements = new Array();
            elements.forEach((element) => {
                metaElements.push(Module.QWPage.createQWElement(element));
            });
            window.act.validateMetaElements(metaElements);
            window.act.executeAtomicRules();
            window.act.executeCompositeRules();
        }, sourceHtmlHeadContent);
        if (!options || !options.rules || options.rules.includes('QW-ACT-R40') || options.rules.includes('59br37')) {
            const viewport = page.viewport();
            await page.setViewport({
                width: 640,
                height: 512
            });
            await page.evaluate(() => {
                window.act.validateZoomedTextNodeNotClippedWithCSSOverflow();
            });
            if (viewport) {
                await page.setViewport(viewport);
            }
        }
        return page.evaluate(() => {
            return window.act.getReport();
        });
    }
    async executeWCAG(page, options, validation) {
        await page.addScriptTag({
            path: require.resolve('@qualweb/wcag-techniques')
        });
        const url = page.url();
        const newTabWasOpen = await this.detectIfUnwantedTabWasOpened(page.browser(), url);
        return await page.evaluate((newTabWasOpen, validation, options) => {
            const wcag = new WCAG.WCAGTechniques(options);
            return wcag.execute(newTabWasOpen, validation);
        }, newTabWasOpen, validation, options);
    }
    async executeBP(page, options) {
        await page.addScriptTag({
            path: require.resolve('@qualweb/best-practices')
        });
        return await page.evaluate((options) => {
            const bp = new BP.BestPractices(options);
            return bp.execute();
        }, options);
    }
    async executeCounter(page) {
        await page.addScriptTag({
            path: require.resolve('@qualweb/counter')
        });
        return await page.evaluate(() => {
            return window.executeCounter();
        });
    }
    async evaluatePage(sourceHtmlHeadContent, page, execute, options, url, validation) {
        const evaluator = await this.getEvaluator(page, url);
        const evaluation = new evaluationRecord_object_1.default(evaluator);
        await this.init(page);
        if (execute.act) {
            evaluation.addModuleEvaluation('act-rules', await this.executeACT(page, sourceHtmlHeadContent, options['act-rules']));
        }
        if (execute.wcag) {
            evaluation.addModuleEvaluation('wcag-techniques', await this.executeWCAG(page, options['wcag-techniques'], validation !== null && validation !== void 0 ? validation : null));
        }
        if (execute.bp) {
            evaluation.addModuleEvaluation('best-practices', await this.executeBP(page, options['best-practices']));
        }
        if (execute.wappalyzer) {
            evaluation.addModuleEvaluation('wappalyzer', await wappalyzer_1.executeWappalyzer(url));
        }
        if (execute.counter) {
            evaluation.addModuleEvaluation('counter', await this.executeCounter(page));
        }
        return evaluation;
    }
    parseUrl(url, pageUrl) {
        const inputUrl = url;
        const completeUrl = pageUrl;
        const protocol = completeUrl.split('://')[0];
        const domainName = completeUrl.split('/')[2];
        const tmp = domainName.split('.');
        const domain = tmp[tmp.length - 1];
        const uri = completeUrl.split('.' + domain)[1];
        return {
            inputUrl,
            protocol,
            domainName,
            domain,
            uri,
            completeUrl
        };
    }
    async detectIfUnwantedTabWasOpened(browser, url) {
        const tabs = await browser.pages();
        let wasOpen = false;
        for (const tab of tabs !== null && tabs !== void 0 ? tabs : []) {
            const target = tab.target();
            const opener = target.opener();
            if (opener) {
                const openerPage = await opener.page();
                if (openerPage && openerPage.url() === url) {
                    wasOpen = true;
                    await tab.close();
                }
            }
        }
        return wasOpen;
    }
}
exports.Evaluation = Evaluation;
//# sourceMappingURL=index.js.map